/**!
* tippy.js v5.0.0-beta.2
* (c) 2017-2019 atomiks
* MIT License
*/
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var popper = require('../../dist/tippy.chunk.cjs.js');
var index = require('../../dist/tippy.chunk.cjs2.js');
require('popper.js');

/**
 * Creates a delegate instance that controls the creation of tippy instances
 * for child elements (`target` CSS selector).
 */
function delegate(targets, props) {
  if (process.env.NODE_ENV !== "production") {
    index.throwErrorWhen(!props || !props.target, index.MISSING_TARGET_WARNING);
  }

  var listeners = [];
  var childTippyInstances = [];
  var target = props.target;
  var nativeProps = popper.removeProperties(props, ['target']);
  var trigger = props.trigger || index.tippy.defaultProps.trigger; // The user needs to specify their own enhanced tippy function to use extra
  // props
  // @ts-ignore

  var tippyConstructor = delegate.tippy || index.tippy;
  var returnValue = tippyConstructor(targets, popper._extends({}, nativeProps, {
    trigger: 'manual'
  }));

  function onTrigger(event) {
    if (event.target) {
      var targetNode = event.target.closest(target);

      if (targetNode) {
        var instance = tippyConstructor(targetNode, popper._extends({}, nativeProps, {
          showOnCreate: true
        }));

        if (instance) {
          childTippyInstances = childTippyInstances.concat(instance);
        }
      }
    }
  }

  function on(element, eventType, listener, options) {
    if (options === void 0) {
      options = false;
    }

    element.addEventListener(eventType, listener, options);
    listeners.push({
      element: element,
      eventType: eventType,
      listener: listener,
      options: options
    });
  }

  function addEventListeners(instance) {
    var reference = instance.reference;
    popper.splitBySpaces(trigger).forEach(function (eventType) {
      switch (eventType) {
        case 'mouseenter':
          {
            on(reference, 'mouseover', onTrigger);
            break;
          }

        case 'focus':
          {
            on(reference, 'focusin', onTrigger);
            break;
          }

        case 'click':
          {
            on(reference, 'click', onTrigger);
          }
      }
    });
  }

  function removeEventListeners() {
    listeners.forEach(function (_ref) {
      var element = _ref.element,
          eventType = _ref.eventType,
          listener = _ref.listener,
          options = _ref.options;
      element.removeEventListener(eventType, listener, options);
    });
    listeners = [];
  }

  function applyMutations(instance) {
    var originalDestroy = instance.destroy;

    instance.destroy = function (shouldDestroyChildInstances) {
      if (shouldDestroyChildInstances === void 0) {
        shouldDestroyChildInstances = true;
      }

      if (shouldDestroyChildInstances) {
        childTippyInstances.forEach(function (instance) {
          instance.destroy();
        });
      }

      childTippyInstances = [];
      removeEventListeners();
      originalDestroy();
    };

    addEventListeners(instance);
    instance.setProps({
      trigger: 'manual'
    });
  }

  var instances = [].concat(returnValue);
  instances.forEach(applyMutations);
  return returnValue;
}

/**
 * Re-uses a single tippy element for many different tippy instances.
 * Replaces v4's `tippy.group()`.
 */
function createSingleton(tippyInstances, optionalProps) {
  if (optionalProps === void 0) {
    optionalProps = {
      delay: 0
    };
  }

  if (process.env.NODE_ENV !== "production") {
    if (!Array.isArray(tippyInstances)) {
      index.throwErrorWhen(!tippyInstances, index.createInvalidCreateSingletonArgumentError(tippyInstances));
      index.throwErrorWhen( // @ts-ignore
      tippyInstances.reference && tippyInstances.reference._tippy, index.ARRAY_MISTAKE_ERROR);
    }

    var isAnyInstancePartOfExistingSingleton = tippyInstances.some(function (instance) {
      return popper.hasOwnProperty(instance, '__singleton');
    });
    index.throwErrorWhen(isAnyInstancePartOfExistingSingleton, index.EXISTING_SINGLETON_ERROR);
    tippyInstances.forEach(function (instance) {
      Object.defineProperty(instance, '__singleton', {
        value: true,
        enumerable: false,
        configurable: true
      });
    });
  } // The user needs to specify their own enhanced tippy function to use extra
  // props
  // @ts-ignore


  var tippyConstructor = createSingleton.tippy || index.tippy;
  var singletonInstance = tippyConstructor(document.createElement('div'));
  var _optionalProps = optionalProps,
      delay = _optionalProps.delay;
  var showTimeout;
  var hideTimeout;

  function clearTimeouts() {
    clearTimeout(showTimeout);
    clearTimeout(hideTimeout);
  }

  tippyInstances.forEach(function (instance) {
    instance.__originalProps = {
      delay: instance.props.delay,
      onShow: instance.props.onShow,
      onTrigger: instance.props.onTrigger
    };
    instance.__originalClearDelayTimeouts = instance.clearDelayTimeouts;

    instance.clearDelayTimeouts = function () {
      instance.__originalClearDelayTimeouts();

      clearTimeouts();
    };

    instance.setProps({
      delay: delay,
      onShow: function onShow() {
        return false;
      },
      onTrigger: function onTrigger(_, event) {
        popper.preserveInvocation(instance.__originalProps.onTrigger, instance.props.onTrigger, [instance, event]);
        singletonInstance.clearDelayTimeouts();
        singletonInstance.setProps(popper._extends({}, instance.props, {
          onShow: instance.__originalProps.onShow,
          triggerTarget: instance.reference
        }));
        var appendTo = instance.props.appendTo;
        singletonInstance.reference.referenceNode = appendTo === 'parent' ? instance.reference.parentNode : popper.invokeWithArgsOrReturn(appendTo, [instance.reference]);

        singletonInstance.reference.getBoundingClientRect = function () {
          return instance.reference.getBoundingClientRect();
        };

        clearTimeouts(); // Edge case: if the tippy is currently hiding (but still mounted and
        // visible due to its opacity), it will slide to the new reference
        // element but fully to fade out before fading back in.
        // Also, we need to ensure the `popper` element doesn't set its
        // `transitionDuration` to 0ms, so it can transition smoothly

        if (!singletonInstance.state.isVisible && singletonInstance.state.isMounted) {
          singletonInstance.show(undefined, false);
        } else {
          showTimeout = setTimeout(function () {
            singletonInstance.show();
          }, popper.getValueAtIndexOrReturn(delay, 0, index.tippy.defaultProps.delay));
        }
      }
    }); // Ensure the lifecycle functions can be updated

    instance.__originalSetProps = instance.setProps;

    instance.setProps = function (partialProps) {
      if (partialProps.onTrigger) {
        instance.__originalProps.onTrigger = partialProps.onTrigger;
      }

      if (partialProps.onShow) {
        instance.__originalProps.onShow = partialProps.onShow;
      }

      if (partialProps.delay !== undefined) {
        instance.__originalProps.delay = partialProps.delay;
      }

      instance.__originalSetProps(popper._extends({}, popper.removeProperties(partialProps, ['onTrigger', 'onShow', 'delay']), {
        delay: delay
      }));
    };
  });
  var originalSetProps = singletonInstance.setProps;

  singletonInstance.setProps = function (partialProps) {
    if (partialProps.delay !== undefined) {
      delay = partialProps.delay;
      tippyInstances.forEach(function (instance) {
        instance.setProps({
          delay: delay
        });
      });
    }

    originalSetProps(partialProps);
  };

  var originalDestroy = singletonInstance.destroy;

  singletonInstance.destroy = function ( // ESLint parser and TypeScript are condradicting each other here for some
  // reason
  // eslint-disable-next-line
  shouldDestroyPassedInstances) {
    if (shouldDestroyPassedInstances === void 0) {
      shouldDestroyPassedInstances = true;
    }

    tippyInstances.forEach(function (instance) {
      if (instance.state.isDestroyed) {
        return;
      } // Restore the instances to their original state


      instance.clearDelayTimeouts = instance.__originalClearDelayTimeouts;
      instance.setProps = instance.__originalSetProps;
      instance.setProps(instance.__originalProps);
      delete instance.__originalClearDelayTimeouts;
      delete instance.__originalProps;
      delete instance.__originalSetProps;

      if (process.env.NODE_ENV !== "production") {
        delete instance.__singleton;
      }

      if (shouldDestroyPassedInstances) {
        instance.destroy();
      }
    });
    originalDestroy();
  };

  return singletonInstance;
}

exports.createSingleton = createSingleton;
exports.delegate = delegate;
//# sourceMappingURL=tippy-addons.cjs.js.map
